Function CountDeletedRecords(cTable)

/*
  The function returns the number of records in cTable marked for deletion
  The function works with dbase tables Level 7 and Level 5. Should also work with 
  Level 3 tables, but not tested.
  Usage:			n = CountDeletedRecords("Fish.dbf") (The table must not be in use!)
  Dependencies: None
  Programmer:	Ivar B. Jessen

  Date posted:	04-05.2012. Posted in group dbase.programming.
  Revised:		05-05-2012. Now returns only nbDeleted. 		
  Revised:		17-05-2012. Changed function name from 'GetDeletedRecs' to
 									'CountDeletedRecords'.
                           Removed errorchecking and cleaned up code. 
*/

   f = new file()
   f.open(cTable, "R")

   // Get number of records in table
   local nRec; nRec = 0
   f.seek(4, 0)
   b4 = asc(f.read(1))
   b5 = asc(f.read(1))
   b6 = asc(f.read(1))
   b7 = asc(f.read(1))
   nRec = b4*2^0 + b5*2^8 + b6*2^16 + b7*2^32

   // Get number of bytes in header
   local nbHead; nbHead = 0
   b8 = asc(f.read(1))
   b9 = asc(f.read(1))
   nbHead = b8*2^0 + b9*2^8
 
   // Getnumber of bytes in record
   local nbRec; nbRec = 0
   b10 = asc(f.read(1))
   b11 = asc(f.read(1))
   nbRec = b10*2^0 + b11*2^8

   // Prepare to run through all records
   local nDeleted; nDeleted = 0

   // Go to beginning of first record 
   f.seek(nbHead, 0)
   if asc(f.read(1)) = 0x2A
      nDeleted ++
   endif

   // Go through the remaining records
   for i = 1 to nRec - 1 
      f.seek(nbRec - 1, 1)
      if asc(f.read(1)) = 0x2A
         nDeleted ++
      endif
   next i

   f.close()

return nDeleted

