/*
   --------------------------------------------------------------------
   Author: Serge Plourde
   Created: 1999.07.20

   Basic purpose:
      -Use to fix a corrupted table
      -Use to copy a vdb level 7 table file with autoinc field, 
       keeping the values of the source autoinc field.

   Basic Process Structure borrowed from FixMemo.prg, by Peter Rorlick, 
   and using DUFLP7\DBF7File.cc (Bowen Moursund) and bFile.cc (Jim Sare) 
   as is.

   Changes to Peter's process:
      - To handle auto increment fields so that the source value is 
        carried to Fixed.dbf.
      - To do this, mixed code had to be used xBase code and OODML. It 
        scans through the source with xBase code, in order to carry out 
        the 'deleted()' record status.  It uses the source OODML source 
        rowset to be able to include also the capacity to handle field 
        names with embedded spaces.
      - It also uses a table def object in order to distinguish the 
        AutoInc field types.  The FixMemo xBase code could not detect 
        that.
      - In addition, it writes a log of the process directly to 
        Corrupt.txt to avoid the trap of crashing if there is not 
        enough memory (if using FixMemo array of corrupted fields 
        descriptions).

      - One very important note from Peter's FixMemo.prg:
         Remember to adjust the default table-level (e.g. 5 or 7) in 
         the BdeAdmin before running this program.

   Modifications:

      January 31, 2000
         Updated to allow use on table with no AutoIncrement field
         The original code by Serge required an AutoIncrement field.
         Solution was to move the check so it surrounded a bit more
         code ... (discovered while fixing a corrupted memo in a table
         with no autoincrement field ...) KJM

      July 5, 2000
         Severe bug fixed by Serge.  The XDML and OODML row pointers now stay in
         sync, even when there are deleted records in the table.

      April 10, 2007
          Paul Van House
          Added use of default database object in order to get full path name of dbf files
          Added code to overcome errors when binary data exists in a BLOB file
          Added code to copy binary data from a BLOB file to "fixed.dbf"

      December 20, 2007
          Minor modifications to avoid having errors with a table that has
          been CONVERTed (i.e., the table has a field named _DBASELOCK
          which does not have the standard field properties, and can hence
          throw an error that is not really an error ...).

   ** Note that this program uses Source Aliasing, specifically the
   ** dUFLP source Alias. If you do not know how to set up
   ** a Source Alias, see the instructions in WHATS.NEW at
   ** the top of the file, or in README.TXT.
   ** Any files referenced by :dUFLP:filename  must be
   ** included in an executable if you are building 
   ** one ... make sure that your project includes these
   ** files.
   **
   ** This particular program uses:
   **           dBF7File.cc


   --------------------------------------------------------------------
*/

param cSourceDBF,oFormRef
local t  //Table definition
local i
local nAutoIncField
local qSource
local qFixed
local nProcessed
local nSourceRows
local lError
local oDBF7File
local nSetAutoIncOpResult
local f  //Progress form
local fLog //For corrupt.txt file
local nFieldsCorrupted
local nRecordsCorrupted
local lRecordError
Local fSize
Local TempFile //pvh 4/10/07
Local TempFolder //pvh 4/10/07
TempFolder = GetEnv('Temp') + "\" //pvh 4/10/07

private cFieldName

if argCount()==0
  Msgbox("Parameter missing - please supply the DBF name (e.g.: FixDbf7('Pa') )")
  return
endif



set talk off
set dele off  && deleted records MUST be visible!
set safe off
set exclusive off
close data

t = new tabledef()
t.tablename := cSourceDBF
t.load()

nAutoIncField = 0
for i = 1 to t.fields.size
  if t.fields[i].fieldName # "_DBASELOCK"
     if t.fields[i].type == 'AUTOINC'	
       nAutoIncField = i
       exit
     endif
  endif
next

*msgbox( iif(nAutoIncField > 0,;
            t.fields[nAutoIncField].fieldname + ' is the autoinc field',;
            'there is no autoInc field' ),;
        'auto inc field msgbox')  //Trace code

//As of 2000.07.05
//To get around the Visual dBASE unability to track "deleted()" status in OODML.
if file( "OODML_Cur.dbf" )
	delete file OODML_Cur.dbf
endif
if file( "OODML_Cur.dbt" )
	delete file OODML_Cur.dbt
endif
if file( "OODML_Cur.mdx" )
	delete file OODML_Cur.mdx
endif
copy table (cSourceDBF) to OODML_Cur
use OODML_Cur
//So that we really can be in sync with the xBase cursor!
recall all
use

use (cSourceDBF)
//Can't do this, if the memo file is corrupted!
copy to temp stru exte
use
crea fixed from temp

nProcessed = 0

set procedure to :dUFLP:DBF7File.CC additive

oDBF7File = new DBF7File( 'Fixed.dbf' )

qSource = new query()
//New as of 2000.07.05, by Serge.
//qSource.sql := "select * from " + cSourceDBF
qSource.sql := [select * from OODML_Cur]

qSource.active := true

qFixed = new query()
qFixed.sql := "select * from fixed"
qFixed.Database = _app.Databases[1] //pvh 4/10/07  so full table path can be retrieved */
qFixed.active := true

f = new form()   // for the progress gauge
f.height := 4
f.text := 'Repairing...'

txProgress = new text(f)
txProgress.text := ''
txProgress.width := f.width
txProgress.Alignment := 1  // Centered
txProgress.top := .5

p = new progress(f)
p.value := 0
p.RangeMax := recc()
p.top := 2
p.left := 1
p.width := f.width - 2

f.open()

use (cSourceDBF)
go top
nSourceRows = reccount()

qSource.rowset.first()  //Make sure both the 'xSource' and the oodml source are in sync.

fLog= new file()
if fLog.exists( "corrupt.txt" )
	fLog.delete( "corrupt.txt" )
endif
fLog.create( "corrupt.txt" )

nFieldsCorrupted = 0
nRecordsCorrupted = 0

fLog.writeLn('FixDBF7 for ' + cSourceDBF + ' started on ' + dtoc(date()) + ' at ' + time())

do while not eof() //xBase source "corrupted" file.
  ++nProcessed
  txProgress.text := 'At record ' + nProcessed +' of '+ nSourceRows
  p.value := nProcessed
  lDeleted = deleted()
  lError = false
  *-We can't append without knowing if the autoincrement field, if any, is valid:
  // Ken's fix was to move the "endif" for the following if statement
  // to a location further down in the code, surrounding more of the
  // code ... 
  if nAutoIncField > 0
    try
	   ? qSource.rowset.fields[nAutoIncField].value
    catch( exception e )
      lError = true   // If an error was thrown, this field must be corrupted. So we'll skip it.
    endtry

     if lError
       qSource.rowset.next()
       skip
       fLog.writeLn( t.fields[nAutoIncField].fieldName + ' is corrupt in record ' + nProcessed )
       fLog.writeLn( '   Record NOT copied, because it needs a valid autoincrement value.' )
       ++nFieldsCorrupted
       ++nRecordsCorrupted
       loop
     endif
     cFieldName = t.fields[nAutoIncField].fieldName

     qFixed.active = false //In order to perform the following statement, we must 'inactivate' it.
     nSetAutoIncOpResult =;
       oDBF7File.SetNextAutoIncValue( cFieldName,;
                                      qSource.rowset.fields[cFieldName].value )

     if nSetAutoIncOpResult <> 0
       qSource.rowset.next()
       skip
       fLog.writeLn( 'Could not add to Fixed.dbf because of unable to set next autoinc field.' )
       fLog.writeLn( '   Reason: ' + { "DBF not found", "Field name not found", "Field not an autoinc field", "Cannot open DBF", "Low level seek failed", "Could not load tableDef object" }[nSetAutoIncOpResult] )
       fLog.writeLn( '   Record NOT copied, because it needs a valid autoincrement value.' )
       ++nFieldsCorrupted
       ++nRecordsCorrupted
       qFixed.active = true
       loop
     endif
  endif // this is where Ken moved the "endif" to ...
 
  qFixed.active = true

  lRecordError = false
  qFixed.rowset.beginAppend()
  FOR i=1 TO t.fields.size
    if i = nAutoIncField
      *-The auto increment field is checked BEFORE a new record is appended.
      loop
    endif
    *
    cFieldName = t.fields[i].FieldName
    if cFieldName == "_DBASELOCK"
       loop
    endif
    lError = false
    try
      /*pvh 4/10/07 added "BINARY" to compare operation */
      if (t.fields[i].type == 'MEMO' OR t.fields[i].type == 'BINARY') and len( qSource.rowset.fields[cFieldName].value ) < 0
        lError = true   // this field must be a corrupted memo
      else
        ?qSource.rowset.fields[cFieldName].value
      endif
    catch( exception e )
      lError = true   // If an error was thrown, this field must be corrupted. So we'll skip it.
    endtry
    *
    if lError
      * This field is corrupt, so we won't write it to FIXED.DBF.
      lRecordError = true
      fLog.writeLn( cFieldName + ' is corrupt in record ' + nProcessed )
	   ++nFieldsCorrupted
    else
      * The field seems OK, so let's write it:
      try
        If  t.fields[i].type == 'BINARY'                                        //pvh 4/10/07
            //Can't just replace binary values..they must be extracted and copied back in but only if there is data //pvh 4/10/07
        		If len( qSource.rowset.fields[cFieldName].value ) > 0              //pvh 4/10/07
               TempFile = Funique(TempFolder + 'dbfix???????.tmp')             //pvh 4/10/07
               qSource.rowset.fields[cFieldName].copyToFile(TempFile)          //pvh 4/10/07
               If File(TempFile)                                               //pvh 4/10/07
               	//File exists...try to import it                             //pvh 4/10/07
               	qFixed.rowset.fields[cFieldName].ReplaceFromFile(TempFile)   //pvh 4/10/07
                  erase (tempfile)                                             //pvh 4/10/07
               EndIf //File(tempFile)                                          //pvh 4/10/07
        		EndIf //len( qSource.rowset.fields[cFieldName].value ) > 0         //pvh 4/10/07
        Else                                                                   //pvh 4/10/07
        	   //if not binary just copy the values
	        qFixed.rowset.fields[cFieldName].value :=;
	           qSource.rowset.fields[cFieldName].value
        EndIf //pvh 4/10/07
      catch( exception e )
        // If an error occurred here, it's probably because this field is an 
        // AUTOINC, which is read-only.  In that case, beware that Autoinc values
        // in Fixed.dbf may not correspond to those from the original table.
        // This could break referential integrity if there are child tables
        // that are related by these Autoinc values.
        lRecordError = true
        fLog.writeLn( 'Unable to set fixed field ' + cFieldName + ' to source ' + cFieldName + "'s value! In record " + nProcessed )
	     ++nFieldsCorrupted
      endtry
    endif
  NEXT i
  if lRecordError
    ++nRecordsCorrupted
  endif
  qFixed.rowset.save()
  // 2000 July 5: This used to be disastrous! Because the way it was handled 
  // before did not take into account that "deleted()" OODML cursor records were
  // "skipped", thus putting the process out of sync.

  //Synchronize both row pointers (xBASE, and OODML)
  if ldeleted
    qFixed.rowset.delete()
  endif
  qSource.rowset.next() 
  skip
enddo

f.close()

qSource.active := false
use  //closes cSourceDBF

fLog.writeLn('   # of corrupted records detected = ' + nRecordsCorrupted )
fLog.writeLn('   # of corrupted fields detected = ' + nFieldsCorrupted )
fLog.writeLn('FixDBF7 for ' + cSourceDBF + ' ended on on ' + dtoc(date()) + ' at ' + time())

msgbox( 'Repaired version of the table is now FIXED.DBF.'+chr(10)+;
        'Corrupted fields have been excluded from FIXED.DBF.'+chr(10)+;
        'A list of these fields is logged in CORRUPT.TXT.',;
        'Repairs done - ' + nRecordsCorrupted + ' records corrupted, ' +;
        nFieldsCorrupted + ' fields corrupted.' )

If type("oFormRef") = "O"

   fSize = fLog.Seek(0,2)
	fLog.Seek(0)
	oFormRef.BadRecords = nRecordsCorrupted
   oFormRef.Editor1.Value = flog.Read(fSize) //fLog.Size(Left(cSourceDBF,RAT('\',cSourceDBF)) + "corrupt.txt"))
   oFormRef.FixedDB  = qFixed.Rowset.TableName
EndIf
fLog.close()
qFixed.active := false

return

