// Metric -- 0 - Chars only
/* -----------------------------------------------------------
   PageXofY.CC
   Author: Ken Mayer
   Date..: 01/29/1998

   This is a custom text control that will
   generate Page x of y, where 'x' is the current page
   and 'y' is the last page number of the report.

   NOTE: If being used in any kind of multi-user situation,
   your report should have all tables in the queries or 
   datamodules locked, as it is possible someone might
   be adding data to the table(s) at the same time you
   are attempting to generate the report (and throwing
   off the page count ...).

   WARNING: This control will (currently) only work as long
   as you do NOT change the name of the pageTemplate, AND
   as long as you do not change the default name of the
   instance of this control placed onto a report -- in other
   words, this must exist:  
                    reportName.pageTemplate1.PageXofY1

   WARNING: For long reports this will take some time, as
   it generates a temporary listing of the report to file,
   in order to get the last page number. At this time, 
   I cannot determine a way to not execute the code to render
   the report when bringing it up in the designer, so you may
   be stuck ...
   ----------------------------------------------------------- */

class PageXofY( parentObj ) of Text( parentObj ) custom
   with ( this )
      height      := 1
      metric      := 0
      text        := class::GetPageNum
      OnOpen      := class::GetLastPage
  endwith

  PROCEDURE GetPageNum
      local nPage
      nPage = 0
      if TYPE("this.form.reportPage") == "N" 
         nPage := this.form.reportPage
      endif
      if type( 'this.LastPage' ) = "U" or this.LastPage = 0
         this.LastPage = 9999
      endif
      return ( "Page "+nPage+" of "+this.LastPage )

  PROCEDURE GetLastPage
      // check to see if this has been set:
      if type( "this.dontLoopOnMe" ) # "U"
         return
      endif

      // have to create an instance of the report:
      cReportName = this.form.className
      rRep = new &cReportName.() // new instance of report

      /*
         -------------------------------------------------------
         find instance of this control, set dontLoopOnMe ...
         assume default (not all controls on report have 'before'
         property, which means we can't loop through controls)
         Have to assume "pageTemplate1" as well as "PageXofY1"
         -------------------------------------------------------
     */
      if type( 'rRep.pageTemplate1.PageXofY1' ) == "O"
         /* 
            Problem: every time you render the report,
            this method will execute, right? If we 
            render it NOW, this method will execute, which
            will attempt to render the report, which will
            cause this method to execute, which will 
            render the report ... nasty. SO, we have to
            have a way to NOT even start that loop:
         */
         rRep.pageTemplate1.PageXofY1.dontLoopOnMe = true
      endif

      // Make some changes:
      rRep.startPage      := 1           // we want all 
      rRep.endPage        := -1          //     pages
      cFileName = fUnique( "last????.prn" )
      rRep.outPutFileName := cFileName   // to temp file
      rRep.outPut         := 2           // printer file   
      nLastPage           = 0            // set counter
      rRep.onPage         := {|| nLastPage++ } // increment for each page

      // Render the report
      rRep.render()

      // Now we're done with this copy of the 
      // report, release it:
      rRep.release()

      // done -- cleanup
      erase ( cFileName )

      // This is what it's all about!
      this.LastPage = nLastPage

EndClass

