/* 
   CloseForms.prg
   Posted by Romain Strieff in the VdB Newsgroups, Jan 19, 2000
   ----------------------------------------------------------------
   Description:

      This utility closes/inspects all dataforms orphaned in memory 
      because of circular references used in the form, which 
      prevent the automatic releasing from memory.

      Examples of such circular references could be:

      Function Form_OnOpen
      form.masterform=form
      etc
 
      Function Form_OnOpen
      form.datamoderef1.ref.myquery1.rowset.fields.parentform=form
      etc

      Function Form_OnOpen
      form.rowset.parentform=form
      etc

      Function Launch_childform_OnClick
      form.child=new form()
      form.child.parent=form
      etc

      You have to stub out these in the OnClose or CanClose
      event of the form if you want the form to release
      itself automatically

   Parameters
      lInspect: if TRUE, the forms are INSPECT()ed instead 
                of closed

   Programmers: 

      Romain Strieff [dBVIPS]

   History: 

      1/19/2000  -- original code

   Usage: 

      to close all forms with circular references 
      (and naturally all others as well ;-)
      orpaned in memory just call it like this:
      
   
      CLOSEFORMS()

      If you just want to inspect the forms to see which forms
      are having such references for debugging purposes, 
      call it with TRUE as parameter

      CLOSEFORMS(true)

   !!!!!!!!!!!!!IMPORTANT REMARK!!!!!!!!!!!!
   this utility will NOT catch any non-data form, there has to be 
   at least 1 rowset in the object tree of the form to be able 
   to find it, since ROWSET is the highest class tat cannot be 
   subclassed from the stock rowset,its name will stay ROWSET
   ---------------------------------------------------------------- 

*/

Function CloseForms(lInspect)
  local a, aR, n
  private f

  //we use a local array to store all the 
  //found rowsets

  aR=new array()

  //find first rowset

  r = findinstance( "rowset" )

  //..and continue finding them until done
  do while not empty( r )
   aR.add(r) //add them to the array
    //...and look for the next one
    r := findinstance( "rowset", r )
  enddo
  //now we loop through the array of rowsets
  for n=1 to aR.size
     //shorten the reference to a private var
     //to be able to use TYPE() on it
     f=aR[n]
     do //endless loop
        try
            //now we check if the object has a sysmenu
            //property which is a property that only
            //forms have (hopefully ;-)
            if type("f.sysmenu") # "U"
               //was the function called with 
               //TRUE as a parameter?
               if lInspect
             
                  inspect(f) //then we inspect it

               else

                  f.release()//if not, we release it

               endif

            endif
            //we try to go one step farther up in the
            //object tree
            f=f.parent
            //if the line above fails ...
        catch (exception e)
            //....there was no more parent this far up, 
            //so we got an exception and landed here
           exit //..so we exit the loop
        endtry

     until false //end of endless loop
  next //next rowset in the array
  release f

return
