/********************************************************************************* 

	Color-Helper-Class
	Author : Marc VdB
   Date	 : 27-04-07

   API-functions that treat colors, use a so called colorref.
   COLORREF in the Windows-System has the form  0x00bbggrr

   The color information in dbase can have a multitude of forms.
   It can be a Hex-value, a Java colorname (like aliceblue) or one 
   of the old DOS-shortcuts (+bg) or one of the Windows-System colornames
   (btntext)

   This class is used to transform a dbase color into a system color,
   that can be used by API-functions.

	Usage : pass a dbase color property to the extract-method of the clas
   		  to separate back and foreground color and retrieve all the RGB 
           values separatly and the COLORREF value for the API-functions

   Example : 	c=new colorado()
				   c.extract(form.colornormal)
   				red=c.back.r   // extracts the red part of the background color
               c_ref = c.fore.colorref  // returns a numeric value, that corresponds 
               								 // to the dbase color and can be passed as is 
                                        // to API functions that use colors...	

   -------------------------------------------------------------
   More details by Ken Mayer:
   To understand this a bit better, this creates two properties:
      fore  -- foreground color
      back  -- background color

   When you call the extract() method, the fore property becomes
   an object reference, which has properties that can be queried:
      c.fore.r        // red
      c.fore.b        // blue
      c.fore.g        // green
      c.fore.colorref // a numeric value, which can be passed
                      // to API functions as noted above.
      c.fore.dbase    // dBASE term for color (if any?)

    The back object will have the same properties that
    can be queried.
   -------------------------------------------------------------

   Special thanks to Rich Muller and Rick Miler for the assocarrays...

**************************************************************************************/

Class Colorado

   this.init()							// create assocarray
	this.fore	= new color(this)	//Foregroundcolor
   this.back	= new color(this)	// Backgroundcolor

   function extract(ccolor)
   	local fo, ba
      ccolor=ltrim(rtrim(ccolor))
      fo = left(ccolor,at('/',ccolor)-1)
      ba = right(ccolor,len(ccolor)-len(fo)-iif('/' $ ccolor,1,0))
		this.fore.set(fo)            // calculate the RGB and COLORREF
      this.back.set(ba)
   return

   function init
   	extern cuint GetSysColor(cint) user32
   	this.co=new assocarray()
      this.co["aliceblue"]           =  16775408
      this.co["antiquewhite"]        =  14150650
      this.co["aqua"]                =  16776960
      this.co["aquamarine"]          =  13959039
      this.co["azure"]               =  16777200
      this.co["beige"]               =  14480885
      this.co["bisque"]              =  12903679
      this.co["black"]               =         0
      this.co["blanchedalmond"]      =  13495295
      this.co["blue"]                =  16711680
      this.co["blueviolet"]          =  14822282
      this.co["brown"]               =   2763429
      this.co["burlywood"]           =   8894686
      this.co["cadetblue"]           =  10526303
      this.co["chartreuse"]          =     65407
      this.co["chocolate"]           =   1993170
      this.co["coral"]               =   5275647
      this.co["cornflowerblue"]      =  15570276
      this.co["cornsilk"]            =  14481663
      this.co["crimson"]             =   3937500
      this.co["cyan"]                =  16776960
      this.co["darkblue"]            =   9109504
      this.co["darkcyan"]            =   9145088
      this.co["darkgoldenrod"]       =    755384
      this.co["darkgray"]            =  11119017
      this.co["darkgreen"]           =     25600
      this.co["darkkhaki"]           =   7059389
      this.co["darkmagenta"]         =   9109643
      this.co["darkolivegreen"]      =   3107669
      this.co["darkorange"]          =     36095
      this.co["darkorchid"]          =  13382297
      this.co["darkred"]             =       139
      this.co["darksalmon"]          =   8034025
      this.co["darkseagreen"]        =   9419919
      this.co["darkslateblue"]       =   9125192
      this.co["darkslategray"]       =   5197615
      this.co["darkturquoise"]       =  13749760
      this.co["darkviolet"]          =  13828244
      this.co["deeppink"]            =   9639167
      this.co["deepskyblue"]         =  16760576
      this.co["dimgray"]             =   6908265
      this.co["dodgerblue"]          =  16748574
      this.co["firebrick"]           =   2237106
      this.co["floralwhite"]         =  15792895
      this.co["forestgreen"]         =   2263842
      this.co["fuchsia"]             =  16711935
      this.co["gainsboro"]           =  14474460
      this.co["ghostwhite"]          =  16775416
      this.co["gold"]                =     55295
      this.co["goldenrod"]           =   2139610
      this.co["gray"]                =   8421504
      this.co["green"]               =     32768
      this.co["greenyellow"]         =   3145645
      this.co["honeydew"]            =  15794160
      this.co["hotpink"]             =  11823615
      this.co["indianred"]           =   6053069
      this.co["indigo"]              =   8519755
      this.co["ivory"]               =  15794175
      this.co["khaki"]               =   9234160
      this.co["lavender"]            =  16443110
      this.co["lavenderblush"]       =  16118015
      this.co["lawngreen"]           =     64636
      this.co["lemonchiffon"]        =  13499135
      this.co["lightblue"]           =  15128749
      this.co["lightcoral"]          =   8421616
      this.co["lightcyan"]           =  16777184
      this.co["lightgoldenrodyellow"]=  13826810
      this.co["lightgreen"]          =   9498256
      this.co["lightgrey"]           =  13882323
      this.co["lightpink"]           =  12695295
      this.co["lightsalmon"]         =   8036607
      this.co["lightseagreen"]       =  11186720
      this.co["lightskyblue"]        =  16436871
      this.co["lightslategray"]      =  10061943
      this.co["lightsteelblue"]      =  14599344
      this.co["lightyellow"]         =  14745599
      this.co["lime"]                =     65280
      this.co["limegreen"]           =   3329330
      this.co["linen"]               =  15134970
      this.co["magenta"]             =  16711935
      this.co["maroon"]              =       128
      this.co["mediumaquamarine"]    =  11193702
      this.co["mediumblue"]          =  13434880
      this.co["mediumorchid"]        =  13850042
      this.co["mediumpurple"]        =  14381203
      this.co["mediumseagreen"]      =   7451452
      this.co["mediumslateblue"]     =  15624315
      this.co["mediumspringgreen"]   =  10156544
      this.co["mediumturquoise"]     =  13422920
      this.co["mediumvioletred"]     =   8721863
      this.co["midnightblue"]        =   7346457
      this.co["mintcream"]           =  16449525
      this.co["mistyrose"]           =  14804223
      this.co["moccasin"]            =  11920639
      this.co["navajowhite"]         =  11394815
      this.co["navy"]                =   8388608
      this.co["oldlace"]             =  15136253
      this.co["olive"]               =     32896
      this.co["olivedrab"]           =   2330219
      this.co["orange"]              =     42495
      this.co["orangered"]           =     17919
      this.co["orchid"]              =  14053594
      this.co["palegoldenrod"]       =  11200750
      this.co["palegreen"]           =  10025880
      this.co["paleturquoise"]       =  15658671
      this.co["palevioletred"]       =   9662683
      this.co["papayawhip"]          =  14020607
      this.co["peachpuff"]           =  12180223
      this.co["peru"]                =   4163021
      this.co["pink"]                =  13353215
      this.co["plum"]                =  14524637
      this.co["powderblue"]          =  15130800
      this.co["purple"]              =   8388736
      this.co["red"]                 =       255
      this.co["rosybrown"]           =   9408444
      this.co["royalblue"]           =  14772545
      this.co["saddlebrown"]         =   1262987
      this.co["salmon"]              =   7504122
      this.co["sandybrown"]          =   6333684
      this.co["seagreen"]            =   5737262
      this.co["seashell"]            =  15660543
      this.co["sienna"]              =   2970272
      this.co["silver"]              =  12632256
      this.co["skyblue"]             =  15453831
      this.co["slateblue"]           =  13458026
      this.co["slategray"]           =   9470064
      this.co["snow"]                =  16448255
      this.co["springgreen"]         =   8388352
      this.co["steelblue"]           =  11829830
      this.co["tan"]                 =   9221330
      this.co["teal"]                =   8421376
      this.co["thistle"]             =  14204888
      this.co["tomato"]              =   4678655
      this.co["turquoise"]           =  13688896
      this.co["violet"]              =  15631086
      this.co["wheat"]               =  11788021
      this.co["white"]               =  16777215
      this.co["whitesmoke"]          =  16119285
      this.co["yellow"]              =     65535
      this.co["yellowgreen"]         =   3329434
      this.co["activeborder"]   		 = GetSysColor(10)
      this.co["activecaption"]  		 = GetSysColor(2)
      this.co["appworkspace"]        = GetSysColor(12)
      this.co["background"]          = GetSysColor(1)
      this.co["btnface"]             = GetSysColor(15)
      this.co["btnhighlight"]        = GetSysColor(20)
      this.co["btnshadow"]           = GetSysColor(16)
      this.co["btntext"]             = GetSysColor(18)
      this.co["captiontext"]         = GetSysColor(9)
      this.co["graytext"]            = GetSysColor(17)
      this.co["highlight"]           = GetSysColor(13)
      this.co["highlighttext"]       = GetSysColor(14)
      this.co["inactiveborder"]      = GetSysColor(11)
      this.co["inactivecaption"]     = GetSysColor(3)
      this.co["inactivecaptiontext"] = GetSysColor(19)
      this.co["infobk"]              = GetSysColor(24)
      this.co["infotext"]            = GetSysColor(23)
      this.co["menu"]                = GetSysColor(4)
      this.co["menutext"]            = GetSysColor(7)
      this.co["scrollbar"]           = GetSysColor(0)
      this.co["window"]              = GetSysColor(5)
      this.co["windowframe"]         = GetSysColor(6)
      this.co["windowtext"]          = GetSysColor(8)
      this.co["b"]   =   8388608    // navy
      this.co["b+"]  =  16711680    // blue
      this.co["b*"]  =  16711680    // blue
      this.co["+b"]  =  16711680    // blue
      this.co["*b"]  =  16711680    // blue
      this.co["g"]   =     32768    // dark green
      this.co["g+"]  =     65280    // lime
      this.co["g*"]  =     65280    // lime
      this.co["+g"]  =     65280    // lime
      this.co["*g"]  =     65280    // lime
      this.co["n"]   =         0    // black
      this.co["n+"]  =   8421504    // gray
      this.co["n*"]  =   8421504    // gray
      this.co["+n"]  =   8421504    // gray
      this.co["*n"]  =   8421504    // gray
      this.co["r"]   =       128    // maroon
      this.co["r+"]  =       255    // red
      this.co["r*"]  =       255    // red
      this.co["+r"]  =       255    // red
      this.co["*r"]  =       255    // red
      this.co["w"]   =  12632256    // silver
      this.co["w+"]  =  16777215    // white
      this.co["w*"]  =  16777215    // white
      this.co["+w"]  =  16777215    // white
      this.co["*w"]  =  16777215    // white
      this.co["bg"]  =   8421376    // teal
      this.co["bg+"] =  16776960    // cyan
      this.co["bg*"] =  16776960    // cyan
      this.co["+bg"] =  16776960    // cyan
      this.co["b+g"] =  16776960    // cyan
      this.co["*bg"] =  16776960    // cyan
      this.co["b*g"] =  16776960    // cyan
      this.co["br"]  =   8388736    // purple
      this.co["br+"] =  16711935    // magenta
      this.co["br*"] =  16711935    // magenta
      this.co["+br"] =  16711935    // magenta
      this.co["*br"] =  16711935    // magenta
      this.co["b+r"] =  16711935    // magenta
      this.co["b*r"] =  16711935    // magenta
      this.co["gb"]  =   8421376    // teal
      this.co["gb+"] =  16776960    // cyan
      this.co["gb*"] =  16776960    // cyan
      this.co["+gb"] =  16776960    // cyan
      this.co["*gb"] =  16776960    // cyan
      this.co["g+b"] =  16776960    // cyan
      this.co["g*b"] =  16776960    // cyan
      this.co["gr"]  =     32896    // olive
      this.co["gr+"] =     65535    // yellow
      this.co["gr*"] =     65535    // yellow
      this.co["+gr"] =     65535    // yellow
      this.co["*gr"] =     65535    // yellow
      this.co["g+r"] =     65535    // yellow
      this.co["g*r"] =     65535    // yellow
      this.co["rb"]  =   8388736    // purple
      this.co["rb+"] =  16711935    // magenta
      this.co["rb*"] =  16711935    // magenta
      this.co["+rb"] =  16711935    // magenta
      this.co["*rb"] =  16711935    // magenta
      this.co["r+b"] =  16711935    // magenta
      this.co["r*b"] =  16711935    // magenta
      this.co["rg"]  =     32896    // olive
      this.co["rg+"] =     65535    // yellow
      this.co["rg*"] =     65535    // yellow
      this.co["+rg"] =     65535    // yellow
      this.co["*rg"] =     65535    // yellow
      this.co["r+g"] =     65535    // yellow
      this.co["r*g"] =     65535    // yellow
   return

endclass

Class Color(oref) 	// holds the RGB and COLORREF values

	this.master = oref // avoid redundancy in the assocarray	
	this.r			= 0 // red
   this.b			= 0 // blue
   this.g			= 0 // green
   this.colorref	= 0
   this.dbase		= "black"

   function set(cdb)
		local b   	
      this.dbase = cdb
   	if empty(this.dbase)
        this.dbase="black"
      endif
      if "0X" $ upper(this.dbase)
      	b=htoi(right("000000"+right(this.dbase,len(this.dbase)-2),6))
      else
         if this.master.co.iskey(lower(this.dbase))
         	b=this.master.co[lower(this.dbase)]
         else
         	b=0
            this.dbase='black'
         endif
		endif		
		this.colorref=this.rgb(b)
   return       

   function rgb(n)
   	local zw,r,g,b
      zw=right("000000"+itoh(n),6)
      r=right(zw,2) ; b=left(zw,2) ; g=substr(zw,3,2)
      this.r =htoi(r) ; this.b =htoi(b) ; this.g =htoi(g)
   return n
endclass
