/*
   kmScheduler.h
   This is a set of definitions of strings used in the forms of
   the Calendar/Scheduler created by Ken Mayer. The idea is to
   place all character strings that are displayed for the user
   in this file as defined constants (using the preprocessor
   command: #DEFINE). In the code for the four different forms,
   anywhere that a character string is displayed, it should be
   replaced by the appropriate constant in the header file.
   This will have the effect of making it possible to have
   folk who know other languages as well as English translate
   the strings, and make this more non-English capable. 

   When  using these, each of the following four forms will need
   to have the command:
       #INCLUDE <kmScheduler_English.h>
   changed to use the appropriate header file, such as:
       #INCLUDE <kmScheduler_German.h>

   The forms are:
       kmScheduler.wfm
       kmSchedEvent.wfm
       km24Time.wfm
       kmSmallCal.wfm

   For those doing the translation, if any of the character strings
   require the use of upper ANSI characters, such as u with an umlaut
   (just an example), rather than typing that in, it may be a good
   idea to use chr( 252 ), as in: "abcde"+chr(252)+"test".
*/ 

// ------------------------------------------------------------------
// For the main calendar form: kmScheduler.wfm
// ------------------------------------------------------------------
// Form's text property:
#DEFINE SCHEDFORMTEXT             "Calendar/Scheduler"

// UI -- User Interface -- speedtips for controls:
#DEFINE MONTHBUTTONTEXT           "Month"
#DEFINE NEXTMONTHBUTTONSPEEDTIP   "Next Month"
#DEFINE PREVMONTHBUTTONSPEEDTIP   "Previous Month"
#DEFINE YEARBUTTONTEXT            "Year"
#DEFINE NEXTYEARBUTTONSPEEDTIP    "Next Year"
#DEFINE PREVYEARBUTTONSPEEDTIP    "Previous Year"
#DEFINE TODAYBUTTONTEXT           "Today"
#DEFINE TODAYBUTTONSPEEDTIP       "Go to today's date"
#DEFINE WEEKVIEWBUTTONTEXT        "Week View"
#DEFINE WEEKVIEWBUTTONSPEEDTIP    "Change to Week View"
#DEFINE WEEKBUTTONTEXT            "Week"
#DEFINE NEXTWEEKBUTTONSPEEDTIP    "Next Week"
#DEFINE PREVWEEKBUTTONSPEEDTIP    "Previous Week"
#DEFINE MONTHVIEWBUTTONTEXT       "Month View"
#DEFINE MONTHVIEWBUTTONSPEEDTIP   "Change to Month View"
#DEFINE MOREBUTTONUPSPEEDTIP      "View previous five items"
#DEFINE MOREBUTTONDOWNSPEEDTIP    "View next five items"

// canOpen event:
#DEFINE OPENFORMERROR             "Cannot open Form"
#DEFINE DATEEXOPENERROR           "Cannot open :dUFLP:DateEx.cc!"
#DEFINE HOLIDAYOPENERROR          "Cannot open :dUFLP:Holiday.cc!"
#DEFINE BDEALIASOPENERROR         "Cannot open :dUFLP:BDEAlias.cc!"
// note the trailing space before the quote -- keep it!
#DEFINE CREATEALIASERROR          "Cannot create BDE Alias: "
#DEFINE CREATETABLEERROR          "Cannot create table: "
#DEFINE TABLELEVELERROR           "Table Level must be 7 -- please change in the BDE Administrator!"
#DEFINE CREATEINDEXESERROR        "Cannot create indexes on table: "
// Holiday names:
#DEFINE ADVENTNAME                "Advent"
#DEFINE EASTERNAME                "Easter"
#DEFINE NEWYEARNAME               "New Year's Day"
#DEFINE VALENTINENAME             "Valentine's Day"
#DEFINE STPATRICKNAME             "St. Patrick's Day"
#DEFINE APRILFOOLNAME             "April Fool's Day"
#DEFINE HALLOWEENNAME             "Halloween"
#DEFINE CHRISTMASNAME             "Christmas"
#DEFINE NEWYEAREVENAME            "New Year's Eve"


// ------------------------------------------------------------------
// kmSchedEvent.wfm
// ------------------------------------------------------------------
// UI Interface:
// Form text:
#DEFINE SCHEDEVENTFORMTEXT        "Individual Event Details"
// Tabbox1 dataSource -- don't translate the word "ARRAY":
#DEFINE TABBOXDATASOURCE          [ARRAY {"Current Event","The Day's Events"}]
// Title text:
#DEFINE TITLETEXTEVENTS           "Event(s)"
// Grid column headings:
#DEFINE GRIDCOLUMN1HEADING        "Time"
#DEFINE GRIDCOLUMN2HEADING        "Event"
#DEFINE GRIDCOLUMN3HEADING        "Repeat"
// Text/TextLabel controls on form:
#DEFINE EVENTTITLETEXT            "Event Title:"
#DEFINE DETAILTEXT                "Detail:"
#DEFINE LOCATIONTEXT              "Location:"
#DEFINE DATETEXT                  "Date:"
#DEFINE STARTTIMETEXT             "Start Time:"
#DEFINE ENDTIMETEXT               "End Time:"
#DEFINE REPEATTYPETEXT            "Repeat Type:"
#DEFINE REPEATEVENTSTEXT          "Repeating Events"
#DEFINE EVENTNOTREPEATTEXT        "This event does not repeat"
#DEFINE STATUSTEXT                "Status:"
// Note the HTML formatting here -- leave those, but move as appropriate:
#DEFINE INFOTEXT                  "<b>Date</b> is required, and defaults "+;
                                  "to the date selected for the event. "+;
                                  "<b>Start Time</b> and <b>End Time</b> "+;
                                  "are required unless this is an 'All Day' "+;
                                  "event. Times must be in 24 hour format "+;
                                  "(if it is after noon, add 12 -- i.e., 1:00 "+;
                                  "pm is 13:00)."
// Checkbox:
#DEFINE ALLDAYCHECKTEXT           "All Day?"
// Buttons:
#DEFINE CHANGEDATEBUTTONSPEEDTIP  "Change Date"
#DEFINE STARTTIMESPEEDTIP         "Change Start Time"
#DEFINE ENDTIMESPEEDTIP           "Change End Time"
// Comboboxes:
#DEFINE REPEATTYPE1               "None"
#DEFINE REPEATTYPE2               "Day"
#DEFINE REPEATTYPE3               "Week"
#DEFINE REPEATTYPE4               "Month"
#DEFINE REPEATTYPE5               "Year"
#DEFINE STATUSCOMBO1              "Open"
#DEFINE STATUSCOMBO2              "Complete"
// On 'repeat' containers:
#DEFINE EVERYTEXT                 "Every"
#DEFINE ENDONTEXT                 "End on:"
#DEFINE REPEATSPINERROR           "Range is 1 to 99"
#DEFINE ENDDATEBUTTONSPEEDTIP     "Change Date"
#DEFINE DAYSTEXT                  "Day(s)"
#DEFINE WEEKSTEXT                 "Week(s)"
#DEFINE MONTHSTEXT                "Month(s)"
#DEFINE YEARSTEXT                 "Year(s)"
#DEFINE DESCRIPTTEXT              "The nth of each month"
#DEFINE REPEATONTEXT              "Repeat on:"
// Delete method:
#DEFINE DELETEQUESTIONTEXT        "Do you wish to delete "+;
                                  "the current event only (Yes), or all occurances of "+;
                                  "this event (No)?"
#DEFINE REPEATINGEVENTTITLE       "Repeating Event"

#DEFINE UNEXPECTEDERRORMSG        "Unexpected error: "
#DEFINE ERRORTITLE                "Error"
// Save Method:
#DEFINE SAVEREPEATQUESTIONTEXT    "Do you wish changes for this event to affect "+;
                                  "the current event only (Yes), or all occurances of "+;
                                  "this event (No)?"
#DEFINE CANNOTSAVETITLE           "Cannot Save!"
#DEFINE TITLEREQUIREDTEXT         "Event Title is required!"

#DEFINE EVENTREQUIREDTEXT         "Date of event is required!"
#DEFINE STARTTIMEREQUIREDTEXT     "Start Time of event is required!"
#DEFINE ENDTIMEREQUIREDTEXT       "End Time of event is required!"
#DEFINE ENDDATEREQUIREDTEXT       "An end date is required for a repeating event!"
#DEFINE DAYOFWEEKREQUIREDTEXT     "At least one day of the week must be selected for a "+;
                                  "'Weekly' Repeating Event!"
// Week Day checkbox change:
#DEFINE WEEKDAYERROR1             "The currently selected date: "
#DEFINE WEEKDAYERROR2             "is a "
#DEFINE WEEKDAYERROR3             ", you cannot unselect this."
#DEFINE WEEKDAYERRORTITLE         "Can't do that"
// TimeEntryfield class:
#DEFINE TIMEENTRYSPEEDTIP         "Enter a time string in 24 hour time"
#DEFINE TIMEENTRYHOURERRORTEXT    "Enter a value from 0-23"
#DEFINE TIMEENTRYHOURERRORTITLE   "Invalid Hour"
#DEFINE TIMEENTRYMINUTEERRORTEXT  "Enter a value from 0 to 59"
#DEFINE TIMEENTRYMINUTEERRORTITLE "Invalid Minutes"



// ------------------------------------------------------------------
// km24Time.wfm
// ------------------------------------------------------------------
#DEFINE TIMEFORMTEXT              "Select Time"
#DEFINE HOURTEXT                  "Hour"
#DEFINE MINUTETEXT                "Minutes"
#DEFINE TIMEFORMDESCRIPTIONTEXT   "Time is in 24 hour format, so for "+;
                                  "any hour after noon, add 12. Example, "+;
                                  "2:00 pm would appear as 14:00. "+;
                                  "Midnight is '00'."
#DEFINE OKBUTTONTEXT              "Ok"
#DEFINE TIMEOKBUTTONSPEEDTIP      "Accept current time"
#DEFINE CANCELBUTTONTEXT          "Cancel"
#DEFINE TIMECANCELBUTTONSPEEDTIP  "Return previously selected value"

// ------------------------------------------------------------------
// kmSmallCal.wfm
// ------------------------------------------------------------------
#DEFINE CALFORMTEXT               "Select Date"
#DEFINE DATEOKBUTTONSPEEDTIP      "Accept current date"

// ------------------------------------------------------------------
// :dUFLP:kenCal3.cc (on kmSmallCal.wfm)
// ------------------------------------------------------------------
#DEFINE KCYEARMINUS10BUTTONSPEEDTIP "10 Years Back"
#DEFINE KCYEARPLUS10BUTTONSPEEDTIP  "10 Years Ahead"
#DEFINE KCPREVYEARBUTTONSPEEDTIP    "Previous Year"
#DEFINE KCNEXTYEARBUTTONSPEEDTIP    "Next Year"
#DEFINE KCMONTHMINUS3BUTTONSPEEDTIP "3 Months Back"
#DEFINE KCMONTHPLUS3BUTTONSPEEDTIP  "3 Months Ahead"
#DEFINE KCPREVMONTHBUTTONSPEEDTIP   "Previous Month"
#DEFINE KCNEXTMONTHBUTTONSPEEDTIP   "Next Month"
#DEFINE KCTODAYBUTTONTEXT           "Today"
#DEFINE KCTODAYBUTTONSPEEDTIP       "Go to Today's Date"

// ------------------------------------------------------------------
// kmSchedColor.wfm
// ------------------------------------------------------------------
// This one is actually in kmScheduler.wfm
#DEFINE CHANGECOLORBUTTONSPEEDTIP     "Change Colors of Buttons"
// The rest are in kmSchedColor.wfm
#DEFINE COLORFORMTITLE                "Change Colors"
#DEFINE HOLIDAYBUTTONTEXT             "Holiday"
#DEFINE CURRENTDAYBUTTONTEXT          "Currently Selected Day"
#DEFINE DEFAULTDAYBUTTONTEXT          "Default"
#DEFINE TEXTCOLORBUTTONTEXT           "T"
#DEFINE BACKGROUNDCOLORBUTTONTEXT     "B"
#DEFINE TEXTCOLORBUTTONSPEEDTIP       "Change color of text"
#DEFINE BACKGROUNDCOLORBUTTONSPEEDTIP "Change color of background"

