// Metric: 6 - Pixels only
/*

	Filename:   NudgeCombo.cc
   Version:    1.10 
   Date:       2002 10 30

   For:        dBASE PLUS / dB2K / Visual dBase 7.** 
               Windows 95/98/ME/XP/NT/2000

   Author:     Ronnie MacGregor
   Contact:    NudgeComboFeedBack@high-lugtonridge.co.uk
	
   
   Revisions:

      2003 02 12  Version 1.10

      Macro instantiation of Nudge Buttons dropped in favour of
      creating them as a property of the ComboBox while still using
      the form as the container. Slightly simpler, and slightly
      faster to open.

      2003 02 14

      Modified to use :duflp: source code alias for bitmap
      images (up.bmp and down.bmp).

         
   Description:

      A "style = 2 // DropDownList" Combobox with nudge buttons
      which allow a single click nudge to the next entry in 
      the list, either up or down, without dropping the box.

      The control prevents doubleClicks onGotFocus and restores
      them onLostFocus. This means that rapid clicking of the nudge
      buttons produces a stream of single click events, and therefore
      the expected number of nudges.


   Usage:
      set procedure to :duflp:nudgecombo.cc additive
      // in the form designer, place the combobox on the design
      // surface.
      // Set datalink and dataSource as you normally would ...

   Methods:

      SetDataSource()

         This method must be used if you wish to re-assign the
         control's DataSource after the control is open. This is
         required to set up an array of the combobox contents for the
         nudge buttons to use.

         For example:

         form.NCB_01.SetDataSource('array {"Item 1","Item 2","Item 3"}')


   Dependancies:

      Up.bmp and Down.bmp

      These tiny bitmaps are required for the nudge buttons, but
      the control has been coded to function error free if they
      are not found.

      Note that when building your application and deploying
      it, the images must either be deployed or compiled into
      the .exe. If deployed, they should either be placed
      in folders referenced by any DEO paths you might have
      set up, or they need to be in the same folder as the .exe.
      

   Acknowledgements:

      Some elements of the Nudge Button instantiation method derived
      from an unrelated sample by Marko Mihorko. 
         
*/



class NudgeCombo(parentObj) of COMBOBOX(parentObj) custom

   // Ensure control drops onto form correctly
   form.metric := 6 // Pixels

   
   // Set up NoDoubleClick capability
   #ifndef UINT
      #define UINT  CUINT
   #endif
   #ifndef VOID
      #define VOID  CVOID
   #endif
   #ifndef BOOL
      #define BOOL  CLOGICAL
   #endif


   if type( "GetDoubleClickTime" ) # "FP"
      extern UINT GetDoubleClickTime(VOID) user32
   endif

   if type( "SetDoubleClickTime" ) # "FP"
      extern BOOL SetDoubleClickTime(UINT) user32
   endif
     
   // Store the system doubleCLick time
   this.OldDoubleClickTime = GetDoubleClickTime()


   // Standard and default properties
   with (this)
      onOpen = CLASS::onOpen
      onGotFocus = {; SetDoubleClickTime(10)}
      onLostFocus = {; SetDoubleClickTime(this.OldDoubleClickTime)}
      canClose = CLASS::canClose
      width = 220
      height = 22
      dataSource = 'array {"Default DataSource 1","Default DataSource 2","Default DataSource 3","Default DataSource 4","Default DataSource 5"}'
      style = 2	// DropDownList
      dropDownHeight = 132
   endwith
   
   
   function onOpen()

      // Create Nudge Up Button
      this.PB_Nudge_Up = new NudgeButton(this.parent)
      with (this.PB_Nudge_Up)
         top = this.top + 2
         left = this.left + this.width - 34
         width = 16
         height = int((this.height - 4)/2)
//         upBitmap = iif(file('up.bmp',true),"filename up.bmp",null)
         try
            upBitmap := "filename :duflp:Up.bmp"
         catch( Exception E )
            upBitmap := null
         endtry
      endwith
      // Give NudgeButton a reference to this control
      this.PB_Nudge_Up.Owner = this
      // Set Nudge Direction
      this.PB_Nudge_Up.Direction = 'up'

      // Create Nudge Down Button
      this.PB_Nudge_Down = new NudgeButton(this.parent)
      with (this.PB_Nudge_Down)
         top = this.top + int(this.height / 2)
         left = this.left + this.width - 34
         width = 16
         height = int( (this.height - 4) / 2)
//         upBitmap = iif(file('down.bmp',true),"filename down.bmp",null)
         try
            upBitmap := "filename :duflp:Down.bmp"
         catch( Exception E )
            upBitmap := null
         endtry
      endwith
      // Give NudgeButton a reference to this control
      this.PB_Nudge_Down.Owner = this
      // Set Nudge Direction
      this.PB_Nudge_Down.Direction = 'down'
      // Deal with odd numbered control heights
      if not ((this.PB_Nudge_Down.height * 2) + 4) == this.height
         this.PB_Nudge_Down.height ++ 
      endif

      // Initialise Nudge array
      this.SetDataSource(this.datasource)

   return



   function SetDataSource(cDSString)
      // Clear existing value
      this.value = ""
      // Assign new DataSource
      this.DataSource = cDSString  
      // Build array of ComboBox contents
      this.aList = new Array()
      cSource = (cDSString)
      do while len(cSource) > 2
         cSource = substr(cSource, at('"',cSource,1) )
         this.aList.add( substr(cSource, 2, (at('"',cSource,2)-2)) )
         cSource = substr(cSource, (at('"',cSource,2)+2) )
      enddo
   return


   function canClose
      // Remove circular references
      this.PB_Nudge_Up.Owner := null
      this.PB_Nudge_Down.Owner := null
   return true

 
endclass

/////////////////////////////////////////////////////////

class NudgeButton(parentObj) of PUSHBUTTON(parentObj) custom

   with (this)
      onClick = CLASS::Nudge
      top = 0
      left = 0
      width = 16
      height = 10
      text = ""
      speedbar = true
   endwith
   if __version__ > 2.5
      // for these toolbuttons, don't use the system theme ...
      this.systemTheme := false // for dBASE Plus 2.6 or later
   endif

   function Nudge()

      // Shorten reference
      to = this.Owner

      // Ensure the NudgeCombo control has focus
      to.setFocus()

      // Initialise
      nCount = 0
      sCurrVal = to.value

      // Find Current value
      do 
         nCount ++
      until sCurrVal == to.aList[nCount]

      // Select new value
      do case

         case this.direction == "up"
            if not nCount == 1
               to.value = to.aList[nCount-1]
            endif

         case this.direction == "down"
            if not nCount == to.aList.size
               to.value = to.aList[nCount+1]
            endif
       
      endcase
      
      // If the control has an onChange event, fire it
      try
         to.onChange()
      catch ( Exception e )
         // do nothing - onChange event doesn't exist
      endtry
            
   return

endclass

/////////////////////////////////////////////////////////

//eof
