/*
   --------------------------------------------------------------//
   Filename.....: speedtext.cc
   ClassName....: speedtext
   Purpose......: Single/Multi-line Speedtip form.
                  One instance can be used for a form or an
                  application.
   Programmer...: Rick Miller
   Date.........: May 29, 2002
   Notes........: Some defines for a few other languages
                  can be found in speedtext.hlp.
                  Not all fonts will automatically size properly.
   Written for..: dB2K 0.1 and newer
   Rev. History.: See Revisions below.
   Dependencies.: applicationIni.cc,
                 :dUFLP:winattr32.cc
   Calls........: None
   Called by....: Any
   Limitations..: up to 30 lines for a Multi-line speedtip.
                  width is subject to getTextExtent().
   Usage........: set procedure to :dUFLP:speedtext.cc additive
   Example......: see below and sample file.
   SampleFile...: speedtext.wfm
   HelpFile.....: speedtext.hlp
   --------------------------------------------------------------//
   Example:
   --------------------------------------------------------
      form.speedtext = new speedtext()
      form.speedtext.open()
      see speedText.wfm for GUI object code.
   --------------------------------------------------------
   Properties:
   --------------------------------------------------------
      above             form relative to object    (default = false)
                           false = below, true = above
      align             form relative to object    (default = 0)
                           0 = Left, 1 = Center, 2 = Right
      inifile           _app.iniFile
                        or
                        _app.exeName - .exe + .ini
                        or
                        id() + ".ini"
      ismdi             use framewin or desktop    (default = true)
                           use framewin for boundary.
      timer.interval    length of display          (default = 2)
      usersave          use inifile ?              (default = true)
                           true = use, false = do not use
   --------------------------------------------------------
   Methods:
   --------------------------------------------------------
      close             close form,
                           remove object from memory
      get               return a property value
      help              display the helpfile when in IDE
      open              makes object able to be displayed
      set               assign a property,
                           return success/fail
      show              show/hide this form,
                           return success/fail
      user_setabove     adjust above via built-in msgbox,
                           return true/false (user made change)
      user_setalignment adjust align via built-in msgbox,
                           return true/false (user made change)
      user_setinterval  adjust timer.interval via built-in msgbox,
                           return true/false (user made change)
      user_setismdi     adjust boundary via built-in msgbox,
                           return true/false (user made change)
   --------------------------------------------------------
   Keywords used for get and set methods:
   --------------------------------------------------------
      above, align, alignh, alignv, fontname, fontsize,;
      gapheight, inifile, interval, ismdi, padcx, padcxfactor,
      padcy, padcyfactor, text, usersave
   --------------------------------------------------------
   Revisions:
   --------------------------------------------------------
      February 18, 2011
         1) default ini file adapted to following
            Windows security model and a standard User
            with the use of applicationIni.cc.
         2) added support for 2 monitors.
      April 08, 2005
         1) fixed bug in x_InitExterns method.
      March 11, 2005
         1) corrected distributed version.
         2) Revised speedtext.hlp (help file).
      February 10, 2005
         1) renamed all external functions.
         2) Revised speedtext.hlp (help file).
      May 14, 2004
         1) added font calls to get & set.
         2) added calculations for height and width when
            changing the default fontName or fontSize.
      December 22, 2003
         1) Revised Documentation.
      December 05, 2003
         1) added form_onMouseMove
         2) Revised speedtext.hlp (help file).
      November 14, 2003
         1) modified showform method.
         2) Added ismdi, padcx, padcy properties.
         3) Added user_setismdi method.
         4) SDI user's should assign the isMdi property to false.
         5) Revised speedtext.hlp (help file).
      October 07, 2003
         1) repair to show method.
*/
//---------------------------------------------------------------//
//          speedtext code starts here.
//---------------------------------------------------------------//
#DEFINE SPMSGSPEEDTEXTABOVE     "Show Speedtip Above objects ?"
#DEFINE SPMSGALIGNMENTTO        "Change Alignment to "
#DEFINE SPMSGCURRENTSETTING     "Current setting is "
#DEFINE SPMSGINCREASESPEEDTEXT  "Increase Speedtip Time ?"
#DEFINE SPMSGDECREASESPEEDTEXT  "Decrease Speedtip Time ?"
#DEFINE SPMSGISMDI              "Use the framewin as Boundary ?"
#DEFINE SPMSGLEFT               "Left"
#DEFINE SPMSGCENTER             "Center"
#DEFINE SPMSGRIGHT              "Right"
#DEFINE SPMSGYES                "Yes"
#DEFINE SPMSGNO                 "No"
#DEFINE SPMSGLINE               "Line: "
#DEFINE SPMSGERROR              "Error: "
// programmer constants.
#define  SPMSG_PUBLISHERINFO    "Rick Miller"
#define  SPMSG_VERSIONINFO      "5.0408"
//
** END HEADER -- do not remove this line
//
// Generated on 05/29/2002
//
parameter bModal
local f
f = new speedtext()
if (bModal)
   f.mdi = false // ensure not MDI
   f.readModal()
else
   f.open()
endif

class speedtext of FORM
   class::x_AssignDefaults()
   with (this)
      canClose = class::FORM_CANCLOSE
      onHelp = class::FORM_ONHELP
      onMouseMove = class::FORM_ONMOUSEMOVE
      onOpen = class::FORM_ONOPEN
      open = class::FORM_OPEN
      metric = 6  // Pixels
      colorNormal = "InfoBk"
      height = 18
      left = 0
      top = 0
      width = 500
      text = ""
      clientEdge = false
      topMost = true
      autoCenter = false
      mdi = false
      smallTitle = true
   endwith

   this.TX2 = new TEXT(this)
   with (this.TX2)
      height = 18
      left = 2
      top = 0
      width = 1000
      text = "speedtext"
      fontname = "MS Sans Serif"
      fontsize = 8
      colorNormal = "InfoText/InfoBk"
   endwith

   this.CT1 = new CONTAINER(this)
   with (this.CT1)
      height = 1
      left = 0
      top = 0
      width = 500
      transparent = true
      colorNormal = "InfoBk"
      borderStyle = 3 // None
      anchor = 2  // Top
   endwith

   this.CT2 = new CONTAINER(this)
   with (this.CT2)
      height = 17
      left = 0
      top = 1
      width = 2
      transparent = true
      colorNormal = "InfoBk"
      borderStyle = 3 // None
      anchor = 3  // Left
   endwith

   this.TX1 = new TEXT(this)
   with (this.TX1)
      onMouseMove = {; this.parent.onmousemove()}
      height = 18
      left = 2
      top = 0
      width = 500
      transparent = true
      text = "speedtext"
      fontname = "MS Sans Serif"
      fontsize = 8
      colorNormal = "InfoText/InfoBk"
      anchor = 6  // container
      alignVertical = 0 // Top
      alignHorizontal = 1 // Center
   endwith


   //------------------------------------------------------------//
   //       end of constructor   -  methods below
   //------------------------------------------------------------//
   // form method.
   function form_canClose
      if (this.usersave)
         //  save settings
         Local cSect
         cSect =  (this.classname)
         class::x_SetIni( ;
                  cSect, "Above", iif((this.above), "1", "0"))
         class::x_SetIni( ;
                  cSect, "Align", "" + (this.align))
         class::x_SetIni( ;
                  cSect, "AlignH", "" + this.tx1.alignHorizontal)
         class::x_SetIni( ;
                  cSect, "AlignV", "" + this.tx1.alignVertical)
         class::x_SetIni( ;
                  cSect, "Ismdi", iif((this.ismdi), "1", "0"))
         class::x_SetIni( ;
                  cSect, "Padcx", "" + (this.padcx))
         class::x_SetIni( ;
                  cSect, "PadcxFactor", "" + (this.padcxfactor))
         class::x_SetIni( ;
                  cSect, "Padcy", "" + (this.padcy))
         class::x_SetIni( ;
                  cSect, "PadcyFactor", "" + (this.padcyfactor))
         class::x_SetIni( ;
                  cSect, "Time", "" + int(this.timer.interval))
         class::x_SetIni( ;
                  cSect, "Gap", "" + int(this.gapheight))
      endif
      // remove timer
      class::timer_ontimer()
      this.timer.oparent := null
      release object this.timer
      this.timer := null
      if type("this.~appIni") == "O"
         this.~appIni.close()
         this.~appIni   := null
      endif
   return   true

   //------------------------------------------------------------//
   // form method.
   function form_onOpen
      // create a winattr32 object to remove the dlgFrame.
      Local oAttr
      set procedure to :dUFLP:winattr32.cc additive
      oAttr =  new winattr32(this)
      oAttr.dlgframe(false)
      // remove user sizing
      oAttr.thickframe(false)
      // destroy the winattr32 object
      oAttr.close()
      oAttr := null
      close procedure :dUFLP:winattr32.cc
      // height requires re-assignment when removing the frame.
      this.height := 18

      // add timer object.
      this.timer = new Timer()
      this.timer.oparent  = this
      this.timer.onTimer  = {; this.oparent.timer_ontimer()}
      this.timer.interval = 2
      this.timer.enabled  = false
      if (this.usersave)
         // retrieve setting from ini file.
         Local n
         n = val(class::x_GetIni((this.classname), "Time"))
         this.timer.interval := iif((n > 0) and (n < 61), n, 2)
      endif
   return

   //------------------------------------------------------------//
   // form method.
   function form_onHelp(p1)
      if not this.~runtime
         Local cFile, cHelp, cProg
         cProg =  program(1)
         cFile =  cProg.left(cProg.lastindexof("."))
         cHelp =  setto("help")
         if file(cFile + ".hlp")
            set help to (cFile)
            if type("argvector(1)") == "C"
               private cTopic
               cTopic = "help " + p1
               &cTopic
               release cTopic
            else
               help intro
            endif
            set help to (cHelp)
         endif
      endif
   return

   //------------------------------------------------------------//
   // form method.
   function form_onMouseMove
      this.show(false)
   return

   //------------------------------------------------------------//
   // overridden form method.
   function form_open
      if not this.indesign
         //  if You open this object in the form designer,
         //  this prevents the form designer from
         //  streaming additional properties.
         class::x_ProtectProperties()
         class::x_ProtectMethods()
         class::x_InitExterns()
         if this.usersave
            class::x_Initialize()
         endif
         this.versioninfo  =  SPMSG_VERSIONINFO
         this.publishinfo  =  SPMSG_PUBLISHERINFO
         this.visible := false
      endif
   return   super::open()

   //------------------------------------------------------------//
   // form method.
   function get(p1)
      Local cKey, pRet
      cKey  =  lower("" + p1)
      pRet  =  ""
      if cKey == "above"
         pRet  := iif(this.above, true, false)
      elseif cKey == "align"
         pRet  := (0 + this.align)
      elseif cKey == "alignh"
         pRet  := int(this.tx1.alignHorizontal)
      elseif cKey == "alignv"
         pRet  := int(this.tx1.alignVertical)
      elseif cKey == "fontname"
         pRet  := ("" + this.tx1.fontName)
      elseif cKey == "fontsize"
         pRet  := (0 + this.tx1.fontSize)
      elseif cKey == "gapheight"
         pRet  := (0 + this.gapheight)
      elseif cKey == "inifile"
         pRet  := ("" + this.inifile)
      elseif cKey == "interval"
         pRet  := (0 + this.timer.interval)
      elseif cKey == "ismdi"
         pRet  := iif(this.ismdi, true, false)
      elseif cKey == "padcx"
         pRet  := (0 + this.padcx)
      elseif cKey == "padcxfactor"
         pRet  := (0 + this.padcxfactor)
      elseif cKey == "padcy"
         pRet  := (0 + this.padcy)
      elseif cKey == "padcyfactor"
         pRet  := (0 + this.padcyfactor)
      elseif cKey == "text"
         pRet  := ("" + this.tx1.text)
      elseif cKey == "usersave"
         pRet  := iif(this.usersave, true, false)
      else
         class::x_ErrorMessage("Get parameter 1 ")
      endif
   return   (pRet)

   //------------------------------------------------------------//
   function inspect
      inspect(this)
   return

   //------------------------------------------------------------//
   function onDesignOpen(bPalette)
      class::x_protectMethods()
      class::x_protectProperties()
   return

   //------------------------------------------------------------//
   // form method.
   function set(p1, p2)
      Local bRet, cKey
      cKey  = lower("" + p1)
      bRet  = false
      if cKey == "above"
         bRet  := class::x_SetBoolProp(p2, this.above)
      elseif cKey == "align"
         bRet  := class::x_SetIntProp(p2, this.align, 3, 0)
      elseif cKey == "alignh"
         bRet  := class::x_SetIntProp(p2, this.tx1.alignHorizontal, 3, 0)
      elseif cKey == "alignv"
         bRet  := class::x_SetIntProp(p2, this.tx1.alignVertical, 3, 0)
      elseif cKey == "fontname"
         if class::x_SetCharProp(p2, this.tx1.fontName)
            bRet  := class::x_SetCharProp(p2, this.tx2.fontName)
         endif
      elseif cKey == "fontsize"
         bRet  := class::x_SetFontSize(p2)
      elseif cKey == "gapheight"
         bRet  := class::x_SetIntProp(p2, this.gapheight, 20, 0)
      elseif cKey == "inifile"
         bRet  := class::x_SetInifile(p2)
      elseif cKey == "interval"
         bRet  := class::x_SetIntProp(p2, this.timer.interval, 61, 2)
      elseif cKey == "ismdi"
         bRet  := class::x_SetBoolProp(p2, this.ismdi)
      elseif cKey == "padcx"
         bRet  := class::x_SetIntProp(p2, this.padcx, 100, 3)
      elseif cKey == "padcxfactor"
         bRet  := class::x_SetFloatProp(p2, this.padcxfactor, 20, 1)
      elseif cKey == "padcy"
         bRet  := class::x_SetIntProp(p2, this.padcy, 100, 3)
      elseif cKey == "padcyfactor"
         bRet  := class::x_SetFloatProp(p2, this.padcyfactor, 20, 1)
      elseif cKey == "text"
         bRet  := class::x_SetCharProp(p2, this.tx1.text)
      elseif cKey == "usersave"
         bRet  := class::x_SetBoolProp(p2, this.usersave)
      else
         class::x_ErrorMessage("Set parameter 1")
      endif
   return iif(bRet, true, false)
   // return true/false (success/fail)

   //------------------------------------------------------------//
   // form method.
   // revised October 07, 2003  -  by Rick Miller
   function show(p1, p2, p3)
      Local bRet
      // call to disable the timer and hide the speedtext.
      bRet  =  class::timer_ontimer()
      if not type("argvector(1)") == "L"
         class::x_ErrorMessage("Show parameter p1")
         bRet := false
      elseif not p1
         // do nothing more.
      elseif type("argvector(2)") == "O"
         class::x_ErrorMessage("Show parameter p2")
         bRet := false
      else
         class::x_FontCheck()
         bRet := class::x_ShowForm(p2, p3)
      endif
   return   iif(bRet, true, false)

   //------------------------------------------------------------//
   // form method.
   function timer_onTimer
      try
         this.timer.enabled   := false
         this.visible   := false
         this.topmost   := false
      catch(exception e)
         class::x_ErrorMessage( ;
                  (e.message) + chr(13) + SPMSGLINE + (e.lineno))
      endtry
   return   iif(this.topmost, false, true)
   // return true/false (success/fail)

   //------------------------------------------------------------//
   // form method.
   function user_setAbove
      class::timer_ontimer()
      Local b, n
      // what is current setting
      b  =  iif(this.above, true, false)
      // ask user about a change
      n  =  msgbox(SPMSGSPEEDTEXTABOVE, SPMSGCURRENTSETTING +;
               iif(b, SPMSGYES, SPMSGNO),;
               iif(b, 35 + 256, 35))
      if n == 6 and not b
         // user wants change to show above
         class::set("above", true)
      elseif n == 7 and b
         // user wants change to show below
         class::set("above", false)
      endif
   return   iif(n > 5, true, false)
   // return true/false (user made change)

   //------------------------------------------------------------//
   // form method.
   function user_setAlign
      class::timer_ontimer()
      Local c1, n1, n2
      c1 =  SPMSGCURRENTSETTING
      n2 =  2
      // what is current setting
      n1 =  int(this.align)
      // ask user about a change
      if n1 == 0
         c1 += SPMSGLEFT
         n2 := msgbox(SPMSGALIGNMENTTO + SPMSGCENTER + " ?",;
                  c1, 35)
         if n2 == 7
            n2 := msgbox(SPMSGALIGNMENTTO + SPMSGRIGHT + " ?",;
                     c1, 35)
            if n2 == 6
               n1 := 1
            endif
         endif
         if n2 == 6
            n1 += 1
         endif
      elseif n1 == 1
         c1 += SPMSGCENTER
         n2 := msgbox(SPMSGALIGNMENTTO + SPMSGLEFT + " ?", c1, 35)
         if n2 == 7
            n2 := msgbox(SPMSGALIGNMENTTO + SPMSGRIGHT + " ?",;
                     c1, 35)
            if n2 == 6
               n1 := 3
            endif
         endif
         if n2 == 6
            n1 -= 1
         endif
      elseif n1 == 2
         c1 += SPMSGRIGHT
         n2 := msgbox(SPMSGALIGNMENTTO + SPMSGLEFT + " ?", c1, 35)
         if n2 == 7
            n2 := msgbox(SPMSGALIGNMENTTO + SPMSGCENTER + " ?",;
                     c1, 35)
            if n2 == 6
               n1 := 3
            endif
         endif
         if n2 == 6
            n1 -= 2
         endif
      endif
      if n2 == 6
         // user answered Yes to 1 of the above
         class::set("align", n1)
      endif
   return   iif(n2 == 6, true, false)
   // return true/false (user made change)

   //------------------------------------------------------------//
   // form method.
   function user_setInterval
      class::timer_ontimer()
      Local c1, n1, n2
      // what is current setting
      n1 =  int(this.timer.interval)
      c1 =  SPMSGCURRENTSETTING + int(n1)
      // ask user about showing longer
      n2 =  msgbox(SPMSGINCREASESPEEDTEXT, c1, 35)
      if n2 == 7
         // does not want longer
         // ask user about showing shorter
         n2 := msgbox(SPMSGDECREASESPEEDTEXT, c1, 35)
         if n2 == 6
            // user wants shorter
            n1 := iif(n1 > 1, n1 - 2, 0)
         endif
      endif
      if n2 == 6
         // user answered Yes to 1 of the above
         class::set("interval", n1 + 1)
      endif
   return   iif(n2 == 6, true, false)
   // return true/false (user made change)

   //------------------------------------------------------------//
   // form method.
   function user_setIsmdi
      class::timer_ontimer()
      Local b, n
      // what is current setting
      b  =  iif(this.ismdi, true, false)
      // ask user about a change
      n  =  msgbox(SPMSGISMDI, SPMSGCURRENTSETTING +;
               iif(b, SPMSGYES, SPMSGNO),;
               iif(b, 35 + 256, 35))
      if n == 6 and not b
         // user wants change to use the framewin.
         class::set("ismdi", true)
      elseif n == 7 and b
         // user wants change to use the desktop.
         class::set("ismdi", false)
      endif
   return   iif(n > 5, true, false)
   // return true/false (user made change)

   //------------------------------------------------------------//
   // form method.
   function x_AssignDefaults
      set procedure to :dUFLP:applicationIni.cc additive
      this.~appIni   =  new applicationIni()
      this.iniFile   =  this.~appIni.value
      // to disable using an ini file,
      // assign the usersave (property) false before
      // the open method is called.
      this.usersave  =  true  // save settings
      this.above  =  false    // false
      this.align  =  0        // 0  =  Left
      this.gapheight =  0     // 0 (pixels)
      this.ismdi  =  true     // true  =  for MDI use.
      // false =  for SDI use.
      this.padcx  =  3        // 3 (pixels)
      this.padcxfactor  =  1  // 1 (multiplier for padcx)
      this.padcy  =  3        // 3 (pixels)
      this.padcyfactor  =  1  // 1 (multiplier for padcy)
      // used for messages when in the IDE.
      this.~runtime  =  iif("run" $ version(0), true, false)
   return

   //------------------------------------------------------------//
   // form method.
   function x_ErrorMessage(p1)
      if not this.~runtime
         // display error when in IDE
         msgbox(SPMSGERROR + p1, SPMSGERROR + "Speedtext.cc", 48)
      endif
   return

   //------------------------------------------------------------//
   function x_FontCheck
      if not (this.tx2.fontName == this.tx1.fontName)
         this.tx2.fontName := this.tx1.fontName
      endif
      if not (this.tx2.fontsize == 8)
         // tx2.fontSize is 8 for calculation purposes.
         this.tx2.fontSize := 8
      endif
   return

   //------------------------------------------------------------//
   // form method.
   // return an int representing a height
   // accounting for a deviation due to fontSize.
   // 13.0 is the default for fontSize = 8.
   function x_GetHeight(nIn)
      Local nRet, nSize
      nSize =  this.tx1.fontSize - 8
      if nIn > 1
         nRet  =  nIn * 13.0
      else
         nRet  =  13.0
      endif
      if nSize > 0 and nSize < 7
         // add on for fontSize above 8 and less than 15.
         // using 8 as the basis.
         nRet  += nRet * (nSize / 8) + nSize / 8
      endif
   return   int(ceiling(nRet))

   //------------------------------------------------------------//
   // form method.
   // read value form ini file
   function x_GetIni(p1, p2)
   return   this.~appIni.getValue(p1, p2, this.inifile)

   //------------------------------------------------------------//
   // form method.
   // return an int from a structure starting with nByte.
   function x_GetLong(sValu, nByte)
      Local nRet
      nRet  =  sValu.getbyte(nByte) + ;
               BitLshift(sValu.getbyte(nByte + 1), 8)  + ;
               BitLshift(sValu.getbyte(nByte + 2), 16) + ;
               BitLshift(sValu.getbyte(nByte + 3), 24)
      if BitSet(nRet, 31)
         nRet  := - bitnot(nRet)
      endif
   return   int(nRet)

   //------------------------------------------------------------//
   // form method.
   // return a float representing a width deviation due to fontSize.
   function x_GetWidth(nIn)
      Local nRet, nSize
      nRet  =  0
      nSize =  this.tx1.fontSize - 8
      if nSize > 0 and nSize < 7
         // add on for fontSize above 8 and less than 15.
         // using 8 as the basis.
         nRet  := nIn * (nSize / 8) + nSize / 8
      endif
   return   nRet

   //------------------------------------------------------------//
   // this method is run during instantiation.
   //------------------------------------------------------------//
   // form method.
   function x_InitExterns
      if not type("RMM_GetClientDesktopRect") == "FP"
         extern CLOGICAL RMM_GetClientDesktopRect( ;
                  CUINT, CUINT, CPTR, CUINT) ;
                  user32 from "SystemParametersInfoA"
      endif
      if not type("RMM_GetSystemMetrics") == "FP"
         extern CINT RMM_GetSystemMetrics(CINT) ;
                  user32 from "GetSystemMetrics"
      endif
      if not type("RMM_GetWindowRect") == "FP"
         extern CLOGICAL RMM_GetWindowRect(CHANDLE, CPTR) ;
                  User32 from "GetWindowRect"
      endif
      if not type("RMM_MoveWindow") == "FP"
         extern CLOGICAL RMM_MoveWindow( ;
                  CHANDLE, CINT, CINT, CINT, CINT, CLOGICAL) ;
                  user32 from "MoveWindow"
      endif
      if not type("RMM_UpdateWindow") == "FP"
         extern CLOGICAL RMM_UpdateWindow(CHANDLE) ;
                  user32 from "UpdateWindow"
      endif
      // the next 3 api calls are not used
      // by this object but are included
      // because any objects that speedtext.cc
      // will be used with will need them available
      if not type("RMM_GetCapture") == "FP"
         extern CHANDLE RMM_GetCapture(CVOID) ;
                  user32 from "GetCapture"
      endif
      if not type("RMM_ReleaseCapture") == "FP"
         extern CLOGICAL RMM_ReleaseCapture(CVOID) ;
                  user32 from "ReleaseCapture"
      endif
      if not type("RMM_SetCapture") == "FP"
         extern CHANDLE RMM_SetCapture(CHANDLE) ;
                  user32 from "SetCapture"
      endif
   return

   //------------------------------------------------------------//
   // form method.
   // this method is called from the form_open
   function x_Initialize
      // retrieve values from inifile.
      Local cSect
      cSect =  (this.classname)
      this.above  := iif(class::x_GetIni( ;
               cSect, "Above") == "1", true, false)
      class::set("align",;
               val(class::x_GetIni(cSect, "Align")))
      class::set("alignh",;
               val(class::x_GetIni(cSect, "AlignH")))
      class::set("alignv",;
               val(class::x_GetIni(cSect, "AlignV")))
      class::set("gapheight",;
               val(class::x_GetIni(cSect, "Gap")))
      this.ismdi  := iif(class::x_GetIni( ;
               cSect, "Ismdi") == "0", false, true)
      class::set("padcx",;
               val(class::x_GetIni(cSect, "Padcx")))
      class::set("padcxfactor",;
               val(class::x_GetIni(cSect, "PadcxFactor")))
      class::set("padcy",;
               val(class::x_GetIni(cSect, "Padcy")))
      class::set("padcyfactor",;
               val(class::x_GetIni(cSect, "PadcyFactor")))
   return

   //------------------------------------------------------------//
   // form method.
   // return an array of strings from the passed text.
   function x_ParseTextChr13(cText)
      Local aRet, cSep, i, nPos1, nPos2
      aRet  =  new array()
      cSep  =  chr(13)
      nPos1 =  0
      for i = 1 to 30
         nPos2 =  cText.indexof(cSep, nPos1)
         if nPos2 == -1
            aRet.add(cText.substring(nPos1, cText.length))
            i  := 35
         else
            aRet.add(cText.substring(nPos1, nPos2))
            nPos1 := nPos2 + 1
         endif
      endfor
   return   aRet

   //------------------------------------------------------------//
   // form method.
   // return an int for the longest string from the passed array.
   function x_ParseTextWidth(aText)
      form.tx2.fontName = form.tx1.fontName
      form.tx2.fontSize = form.tx1.fontSize
      Local cTemp, i, n1, nRet
      n1 =  0
      nRet  =  0
      for i = 1 to aText.size
         n1 := form.tx2.getTextExtent("" + aText[i])
         if n1 > nRet
            nRet := int(n1)
         endif
      endfor
   return   int(ceiling(nRet + class::x_GetWidth(nRet)))

   //------------------------------------------------------------//
   // form method.
   function x_ProtectMethods
      // protected methods.
      protect  onDesignOpen, onOpen
      protect  x_AssignDefaults, x_ErrorMessage, x_FontCheck,;
               x_GetHeight, x_GetIni, x_GetLong, x_GetWidth,;
               x_InitExterns, x_Initialize, x_ParseTextChr13,;
               x_ParseTextWidth, x_ProtectMethods,;
               x_ProtectProperties, x_SetBoolProp, x_SetCharProp,;
               x_SetFloatProp, x_SetFontSize, x_SetIni,;
               x_SetInifile, x_SetIntProp, x_ShowForm
   return

   //------------------------------------------------------------//
   // form method.
   function x_ProtectProperties
      // protected properties.
      protect  ~appIni, ~runtime, ct1, ct2, gapheight,;
               padcx, padcxfactor, padcy, padcyfactor,;
               tx1, tx2
   return

   //------------------------------------------------------------//
   // form method.
   function x_SetBoolProp(bVal, oProp)
      Local bRet
      bRet  =  type("argvector(1)") == "L"
      if bRet
         oProp := iif(bVal, true, false)
      endif
   return   iif(bRet, true, false)
   // return true/false (success/fail)

   //------------------------------------------------------------//
   // form method.
   function x_SetCharProp(cVal, oProp)
      Local bRet
      bRet  =  type("argvector(1)") == "C"
      if bRet
         oProp := "" + cVal
      endif
   return   iif(bRet, true, false)
   // return true/false (success/fail)

   //------------------------------------------------------------//
   // form method.
   // generic function for assigning a property for
   // properties that use a float.
   function x_SetFloatProp(nVal, oProp, nMod, nDef)
      Local bRet
      bRet  =  type("argVector(1)") == "N"
      if bRet
         Local n
         n  =  mod(nVal, nMod)
         oProp := iif(n == 0, nDef, n)
      endif
   return   iif(bRet, true, false)
   // return true/false (success/fail)

   //------------------------------------------------------------//
   // form method.
   function x_SetFontSize(p1)
      Local bRet
      bRet  =  false
      if type("argvector(1)") == "N"
         // if p1 > 7 and p1 < 15
         if p1 > 7
            this.tx1.fontSize := p1
            bRet  := true
            this.tx2.fontSize := p1
         endif
      endif
   return   iif(bRet, true, false)
   // return true/false (success/fail)

   //------------------------------------------------------------//
   // form method.
   // write value to ini file
   function x_SetIni(p1, p2, p3)
   return   this.~appIni.setValue(p1, p2, p3, this.inifile)

   //------------------------------------------------------------//
   // form method.
   function x_SetInifile(p1)
      Local bRet
      bRet  =  false
      if type("argvector(1)") == "C"
         if this.~appIni.fileCheck(p1)
            this.inifile   := p1
            bRet  := true
         endif
      endif
   return   iif(bRet, true, false)
   // return true/false (success/fail)

   //------------------------------------------------------------//
   // form method.
   // generic function for assigning a property for
   // properties that use an integer.
   function x_SetIntProp(nInt, oProp, nMod, nDef)
      Local bRet
      bRet  =  type("argVector(1)") == "N"
      if bRet
         Local n
         n  =  int(mod(abs(nInt), nMod))
         oProp := iif(n == 0, nDef, n)
      endif
   return   iif(bRet, true, false)
   // return true/false (success/fail)

   //------------------------------------------------------------//
   // form method.
   // nHwnd =  (0 + object.hwnd) or Left (int).
   // nTop  =  false (use the passed nHwnd) or Top (int).
   function x_Showform(nHwnd, nTop)
      this.top := RMM_GetSYstemMetrics(1) + 10
      this.visible   := true
      // display the speedtext form.
      // all Local variables for this method.
      // vars beginning with "o"  from and for this object.
      // vars beginning with "nA" are from
      //    the _app.framewin or the desktop position.
      // vars beginning with "nH" from the passed hWnd rect.
      Local aText, bTop,;
               nAbot, nAhig, nAlef, nArig, nAtop, nAwid,;
               nHbot, nHhig, nHlef, nHrig, nHtop, nHwid,;
               oAbov, oAlgn, oGapy, oHigh, oLeft,;
               oTop, oWide,;
               sRect
      bTop  =  iif(type("argvector(2)") == "N", true, false)
      oAbov =  iif(this.above, true, false)
      oAlgn =  (0 + this.align)
      oLeft =  0
      oGapy =  (0 + this.gapheight)
      oTop  =  0

      // place up to the first 30 lines into an array.
      aText =  class::x_ParseTextChr13(this.tx1.text)
      // calc the width of the longest string.
      oWide =  class::x_ParseTextWidth(aText)
      // add on width for the frames, ct2, and the width padding.
      oWide += RMM_GetSystemMetrics(5) * 2 +;
               (this.ct2.width * 2) +;
               (this.padcx * this.padcxfactor)
      // calc the height for the number of strings.
      oHigh =  class::x_GetHeight(int(aText.size))
      // add on height for the frames, ct1, and the height padding.
      oHigh += RMM_GetSystemMetrics(6) * 2 +;
               this.ct1.height +;
               (this.padcy * this.padcyfactor)
      release object aText
      aText := null

      // get the boundary values to use.
      sRect =  replicate(chr(0), 8)
      if this.ismdi
         // use the framewin for a boundary.
         RMM_GetWindowRect(_app.framewin.hwnd, sRect)
      else
         // use the desktop for a boundary.
         RMM_GetClientDesktopRect(48, 0, sRect, 0)
      endif
      nAlef =  int(class::x_GetLong(sRect, 0 ))
      nAtop =  int(class::x_GetLong(sRect, 4 ))
      nArig =  int(class::x_GetLong(sRect, 8 ))
      nAbot =  int(class::x_GetLong(sRect, 12))
      nAwid =  (nArig - nAlef)
      nAhig =  (nAbot - nAtop)

      // get the top position.
      if not bTop
         // position with the passed hWnd (nHwnd).
         sRect := replicate(chr(0), 8)
         RMM_GetWindowRect(nHwnd, sRect)
         nHlef =  int(class::x_GetLong(sRect, 0 ))
         nHtop =  int(class::x_GetLong(sRect, 4 ))
         nHrig =  int(class::x_GetLong(sRect, 8 ))
         nHbot =  int(class::x_GetLong(sRect, 12))
         nHwid =  int(nHrig - nHlef)
         nHhig =  int(nHbot - nHtop)
         if oAbov
            oTop  := int(nHtop - oHigh - oGapy)
         else
            oTop  := int(nHbot + oGapy)
         endif
      endif

      // get the left position.
      if bTop
         // position with the passed nHwnd (left) value.
         oLeft := (0 + nHwnd)
         // position with passed nTop value.
         oTop  := int(nTop)
      elseif oAlgn == 1
         // alignHorizontal to the center of the passed hWnd.
         oLeft := (nHwid - oWide) / 2 + nHlef
      elseif oAlgn == 2
         // alignHorizontal to the right of the passed hWnd.
         oLeft := (nHrig - oWide)
      else
         // alignHorizontal to the left of the passed hWnd.
         oLeft := nHlef
      endif

      if not bTop
         if not this.isMdi
            if RMM_GetSystemMetrics(80) > 1
               Local    nPvx, nPvy, nPx, nPy
               // get primary screen size.
               // primary screen width.
               nPx   =  RMM_GetSystemMetrics(0)
               // primary screen height.
               nPy   =  RMM_GetSystemMetrics(1)
               // get the total virtual size.
               // the virtual width is the total of the 2 screens.
               // virtual width.
               nPvx  =  RMM_GetSystemMetrics(78)
               // the virtual height is the larger of the 2 screens.
               nPvy  =  RMM_GetSystemMetrics(79)
               // determine which screen is in use.
               // get hWndParent position.
               if nHlef > nPx - RMM_GetSystemMetrics(32)
                  // using the 2nd screen.
                  // the 2nd screen is positioned to the right.
                  nAlef := int(nPx)
                  nArig := int(nPvx)
                  nAbot := int(nPvy)
               endif
            endif
         endif
      endif
      // insure speedtext is on the framewin or desktop
      // by re-assigning the top or left if necessary.
      if oAbov
         if oTop < nAtop
            // will be too high.
            oTop  := nAtop
         endif
      else
         if (oTop + oHigh) > nAbot
            // will be too low.
            oTop  := (nAbot - oHigh)
         endif
      endif
      if (oLeft + oWide) > nArig
         // will be too far right.
         oLeft := (nArig - oWide)
      elseif oLeft < nAlef
         // will be too far left.
         oLeft := nAlef
      endif

      // display speedtext to user.
      RMM_MoveWindow(this.hwnd, oLeft, oTop, oWide, oHigh, true)
      this.visible   := true
      this.topmost   := true
      // turn the timer on.
      this.timer.enabled   := true
   return   iif(this.visible, true, false)
   // return true/false (success/fail)

endclass
//---------------------------------------------------------------//
//          end of speedtext
//---------------------------------------------------------------//
